(declare (unit bindings))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
<#

;*************************
; HELP
;*************************
;
; In this file because what we show in the help is based on the key
; bindings.

; The available things to show help about
(define help-items
  '(
    (q "q: quit")
    (C-coll "C: show collection(s)")
    (P "P: to/from current playlist")
    (left "left=h: back out")
    (right "right=l=ret=tab: enter/use selection")
    (right-play "right=l=ret=tab: play song")
    (right-add "a=right=l=ret=tab: add song(s) to playlist")
    (right-col "a=right=l=ret=tab: add column(s)")
    (right-tags "a=right=l=ret=tab: add tags(s) to song")
    (a "a: add song(s) to playlist")
    (A-dir "A: add song(s) to media library")
    (m "m: main menu")
    (down "down=j=PgDn: scroll down")
    (up "up=k=PgUp: scroll up")
    (r "r: remove song(s)")
    (r-col "r: remove column(s)")
    (r-tags "r: remove tag(s) from song")
    (p "p: previous song")
    (n "n: next song")
    (s "s: start play")
    (e "e: end play")
    (E "E: edit song info")
    (A "A: add all songs to the playlist")
    (A-props "A: add property")
    (ctrl-p "CTRL-p: play/pause")
    (space "SPACE: mark song")
    (f "f: go to filter screen")
    (plus "+=u: move song up")
    (minus "-=d: move song down")
    (plus-col "+=u: move column up")
    (minus-col "-=d: move column down")
    (write "W: toggle prop->file writing")
    (U-play "U: use playlist")
    (D-play "D: delete playlist")
    (N-play "N: create a new playlist")
    (N-coll "N: store as new collection")
    (D-coll "D: delete collection")
    (N-dir "N: create a new directory")
    (M-dir "M: modify file/directory name")
    (S "S: shuffle")
    (T "T: manage song tags")
    (c "c: cut file(s)/directory(s)")
    (p "p: paste")
    (ctrl-r "CTRL-r: auto-rating")
    (ctrl-k "CTRL-k: auto-sKip")
    (H "H: big help")
    (question "?: toggle small help")
    (R "R: rate song")
    (L "L: loop all")
    (O "O: loop one")
    (J "J: jump to playing song dir")
    (D "D: delete song file")
    (o "o: cOpy song to other playlist")
    ))

; Complete list of current help settings, for use by the long help
(define current-help #f)

; Show help on a given set of items
(define (show-little-help . items)
  (set! current-help (flatten items common-help))
  (cond
    [(and
       help-widget
       little-help)
     (let*
       [(strings
	  (string-split
	    (fmt #f (fit
		      maxwidth
		      (with-width
			maxwidth
			(wrap-lines
			  (join
			    dsp
			    (map
			      (lambda (x) (second (assoc x help-items)))
			      ; Put common-to-all items here.
			      (flatten items common-help))
			    " / ")))))
	    "\n"))]
       (setCDKLabelMessage
	 help-widget
	 (make_strings strings)
	 (length strings))

       (drawCDKLabel help-widget 1))]))

; Do the same help information as a popup label
(define (show-big-help)
  (let*
    ([label (newCDKLabel
	      cdk_screen			; CDKSCREEN *cdkscreen,
	      10				; int xpos,
	      10				; int ypos,
	      (make_strings			; char **message,
		(map
		  (lambda (x) (second (assoc x help-items)))
		  current-help))
	      (length current-help)		; int messageLines,
	      1					; boolean box,
	      0)])				; boolean shadow);

    (drawCDKLabel label 1)
    (waitCDKLabel label (foreign-value "(char) 0" char))))

; Help bits that are common to all screens.
(define common-help
  (list 'question 'H 'down 'up 'm 'f 's 'e 'L 'O 'space 'write 'ctrl-p 'ctrl-r 'ctrl-k 'C-coll 'P 'q))
(define song-views-help
  (list 'a 'n 'p 'r 'J 'R 'T 'A 'E 'o))
; Individual help defines
(define (show-current-playlist-help)
  (show-little-help
    (append
      (list 'S 'plus 'minus 'left 'right-play)
      song-views-help)))
(define (show-collections-help)
  (show-little-help 'a 'D-coll 'left 'right))
(define (show-collection-help)
  (show-little-help
    (append
      (list 'D-coll 'left 'right-add)
      song-views-help)))
(define (show-menu-help)
  (show-little-help 'right))
(define (show-columns-help)
  (show-little-help 'r-col 'plus-col 'minus-col 'left 'right-col))
(define (show-playlists-help)
  (show-little-help 'a 'r 'N-play 'U-play 'D-play 'left 'right))
(define (show-playlist-help)
  (show-little-help
    (append
      (list 'N-play 'U-play 'D-play 'left 'right-add)
      song-views-help)))
(define (show-all-music-help)
  (show-little-help
    (append
      (list 'right-add)
      song-views-help)))
(define (show-filter-help)
  (show-little-help
    (append
      (list 'N-coll 'right-add)
      song-views-help)))
(define (show-props-help)
  (show-little-help 'A-props))
(define (show-tags-help)
  (show-little-help 'r-tags 'left 'right-tags))
(define (show-ratings-help)
  (show-little-help 'left 'right))
(define (show-directory-help)
  (show-little-help
    (append
      (list 'A-dir 'N-dir 'M-dir 'c 'p 'D 'left 'right 'o)
      song-views-help)))

;*************************
; BINDING FUNCTIONS
;*************************

(define-macro
  (mtrace x)
  `(begin 
     (display (conc "Trace:\n    " ,x "\n"))
     ,x))

(define-macro
  (binding key c-key . forms)
  ;(mtrace
  `(begin
     (define-external
       (,(string->symbol (conc (symbol->string key) "_binding"))
	 (unsigned-int cdktype)
	 (c-pointer object)
	 (c-pointer clientData)
	 (char key))
       int
       (pre-binding ',key)
       ,@forms
       (post-binding ',key)
       1)
     (bindCDKObject
       (foreign-value "vSELECTION" unsigned-int)
       selection-widget
       ,c-key
       (location ,(string->symbol (conc (symbol->string key) "_binding")))
       #f)));)

(define (make-focus-key)
  (cond
    [(and (eq? system-state 'collections) (eq? collection-type 'playlist))
     (conc system-state collection-type current-playlist)]
    [(and (eq? system-state 'collections) (eq? collection-type 'collection))
     (conc system-state collection-type current-collection)]
    [(and (eq? system-state 'directory))
     (conc system-state directory-current)]
    [else (->string system-state)]))

; This is run before all binding code.
(define
  (pre-binding key)
  (logger "In " key " binding." nl)
  ; Reset status-extra; the actual binding will set it if necessary.
  (global-set! 'status-extra #f)

  ; Store the focus line
  (define-external sw c-pointer selection-widget)
  (let*
    (
     ; Set up a fairly reliable key
     [key (make-focus-key)]
     [line (getCDKSelectionCurrent selection-widget)])
    (global-set!
      'focus-alist
      (alist-cons
	key
	line
	(alist-delete
	  key
	  focus-alist))))
  (logger "In " key " binding; state: " system-state ", focus-alist: " focus-alist nl))

; Put the focus back to where it was
(define (reset-focus)
  (setCDKSelectionCurrent
    selection-widget
    (cdr
      ; Get this list's focus location or (0.0)
      (or
	(assoc (make-focus-key) focus-alist)
	(cons 0 0)))))

; This is run after all binding code.
(define
  (post-binding key)
  (logger "Exiting " key " binding." nl)
  (reset-focus)
  (logger "Still exiting " key " binding." nl)
  ; Update the focus line
  (drawCDKSelection selection-widget 1)
  (logger "Done with " key " binding." nl))

; Converts keystrokes to other keystrokes before processing; used in
; particular to turn j and k into arrow keys, and rebind RETURN, TAB
(define-external
  (key_munge (unsigned-int cdktype)
	     (c-pointer object)
	     (c-pointer clientData)
	     (unsigned-int key))
  int
  (logger "In key_munge: " key nl)
  (cond
    [(eq? key (char->uint #\j))
     (logger "key_munge got j arrow.\n")
     (injectCDKSelection object (foreign-value "KEY_DOWN" unsigned-int))
     0]
    [(eq? key (char->uint #\k))
     (logger "key_munge got k arrow.\n")
     (injectCDKSelection object (foreign-value "KEY_UP" unsigned-int))
     0]
    [(eq? key (char->uint #\h))
     (logger "key_munge got h arrow.\n")
     (injectCDKSelection object (foreign-value "KEY_LEFT" unsigned-int))
     0]
    [(eq? key (char->uint #\l))
     (logger "key_munge got l arrow.\n")
     (injectCDKSelection object (foreign-value "KEY_RIGHT" unsigned-int))
     0]
    [(eq? key (foreign-value "KEY_ENTER" unsigned-int))
     (logger "key_munge got return.\n")
     (injectCDKSelection object (foreign-value "KEY_RIGHT" unsigned-int))
     0]
    [(eq? key (foreign-value "KEY_TAB" unsigned-int))
     (logger "key_munge got tab.\n")
     (injectCDKSelection object (foreign-value "KEY_RIGHT" unsigned-int))
     0]
    [else
      (logger "key_munge no match.\n")
      1]))

; For things that work in any song view.
(define (song-views run)
  (logger "In song-views: " system-state ", " collection-type ", " current-playlist ", " current-collection nl)
  (cond
    ; See if we're in a collection view that has a
    ; current item set (i.e. one in which what's under
    ; the cursor is a song)
    [(and
       (eq? system-state 'collections)
       (or
	 (and
	   (eq? collection-type 'playlist)
	   (not (eq? current-playlist #f)))
	 (and
	   (eq? collection-type 'collection)
	   (not (eq? current-collection #f)))))

     (logger "In song-views: running.\n")
     (run)]
    ; Or in the all-music view
    [(eq? system-state 'all-music)
     (run)]
    ; Or in the filter view
    [(eq? system-state 'filter)
     (run)]
    ; Or in the directory view
    [(eq? system-state 'directory)
     (run)]
    ; Or in the current playlist view
    [(eq? system-state 'current-playlist)
     (run)]))

; Turn a Scheme charachter into a C unsigned int for passing to
; other C calls
(define (char->uint mychar)
  (logger "In char->uint: " mychar "\n")
  (define-external mychar2 char mychar)
  (foreign-value "(unsigned int) mychar2" unsigned-int))

; Set up the bindings
(define (set-bindings)
  ; Clean user-define bindings out
  ;(cleanCDKObjectBindings (foreign-value "vSELECTION" unsigned-int) selection-widget)

  ; Preprocess to convert some keystrokes to others
  (setCDKSelectionPreProcess selection-widget (location key_munge) #f)

  ; p key:
  ;
  ; 	Play previous song
  ;
  ; Directory View:
  ;
  ; 	Paste items
  (binding p (char->uint #\p)
	   (cond
	     [(eq? system-state 'directory)
	      (directory-paste)]
	     [else
	       (playlist-previous)]))

  ; n key:
  ;
  ; 	Play next song
  (binding n (char->uint #\n)
	   (playlist-next))

  ; s key:
  ;
  ; 	Start play.
  (binding s (char->uint #\s)
	   (my-xmms-wait (xmmsc_playback_start conn))
	   (refresh-current))

  ; e key:
  ;
  ; 	End play.
  (binding e (char->uint #\e)
	   (my-xmms-wait (xmmsc_playback_stop conn))
	   (refresh-current))

  ; CTRL-p key:
  ;
  ; 	Play/pause toggle.
  (binding ctrl_p (foreign-value "CTRL('p')" unsigned-int)
	   (let*
	     ([status (status-get-playback-status)])
	     (cond
	       [(= 1 status)
		(my-xmms-wait (xmmsc_playback_pause conn))]
	       [else
		 (my-xmms-wait (xmmsc_playback_start conn))])
	     (refresh-current)))

  ; CTRL-r key:
  ;
  ; 	Auto-rating
  (binding ctrl_r (foreign-value "CTRL('r')" unsigned-int)
	   (ratings-auto))

  ; CTRL-k key:
  ;
  ; 	Auto-sKip
  (binding ctrl_k (foreign-value "CTRL('k')" unsigned-int)
	   (ratings-auto-skip-toggle))

  ; + key:
  ;
  ; Current Playlist:
  ; 	
  ; 	Move a song up in the list
  ;
  ; Columns:
  ;
  ; 	Move a columns up in the list
  (binding plus (char->uint #\+)
	   (cond
	     [(eq? system-state 'current-playlist)
	      (playlist-up object)]
	     [(eq? system-state 'columns)
	      (columns-up object)]))

  ; - key:
  ;
  ; Current Playlist:
  ; 	
  ; 	Move a song down in the list
  ;
  ; Columns:
  ;
  ; 	Move a columns down in the list
  (binding minus (char->uint #\-)
	   (cond
	     [(eq? system-state 'current-playlist)
	      (playlist-down object)]
	     [(eq? system-state 'columns)
	      (columns-down object)]))

  ; d key:
  ;
  ; Call minus
  (binding d (char->uint #\d)
	   (minus_binding cdktype object clientData key))

  ; o key:
  ;
  ; Copy song to other playlist
  (binding o (char->uint #\o)
	   (cond
	     [(eq? system-state 'directory)
	      (playlist-other-add object)]
	     [else
	       (song-views (lambda () (playlist-other-add object)))]))

  ; u key:
  ;
  ; Call plus
  (binding u (char->uint #\u)
	   (plus_binding cdktype object clientData key))

  ; c key:
  ;
  ; Directory View:
  ;
  ; 	Cut item(s)
  (binding c (char->uint #\c)
	   (cond
	     [(eq? system-state 'directory)
	      (directory-cut object)]))

  ; r key:
  ;
  ; Others:
  ;
  ; 	Remove the current item(s) from the current playlist.
  ;
  ; Columns:
  ; 	
  ; 	Remove the current item(s) from the column list.
  ; 
  ; Tags:
  ;
  ; 	Remove the current tag(s) from the song
  (binding r (char->uint #\r)
	   (cond
	     [(eq? system-state 'columns)
	      (columns-remove object)]
	     [(eq? system-state 'tags)
	      (tags-remove object)]
	     [else
	       (playlist-remove object)]))

  ; a key:
  ;
  ; Columns:
  ; 	
  ; 	Add the current item(s) to the column list.
  ;
  ; Tags:
  ; 	
  ; 	Add the current tag(s) to the song.
  ;
  ; All Others But Current Playlist:
  ;
  ; 	Add the current item(s) to the current playlist.
  ;
  (binding a (char->uint #\a)
	   (cond
	     [(eq? system-state 'columns)
	      (columns-add object)]
	     [(eq? system-state 'tags)
	      (tags-add object)]
	     [(eq? system-state 'directory)
	      (directory-playlist-add object)]
	     [(not (eq? system-state 'current-playlist))
	      (playlist-add object)]))

  ; m key:
  ;
  ; Show the menu
  (binding m (char->uint #\m)
	   (show-menu))

  ; f key:
  ;
  ; 	Filter all-music (and go to the result)
  (binding f (char->uint #\f)
	   (menu-selection 'filter))

  ; J key:
  ;
  ; Song views:
  ;
  ; 	Jump to song dir
  (binding J (char->uint #\J)
	   (global-set! 'directory-current current-song-directory)
	   (menu-selection 'directory))

  ; E key:
  ;
  ; Song views:
  ;
  ; 	Edit song info.
  (binding E (char->uint #\E)
	   (song-views (lambda () (props-display object))))

  ; A key:
  ;
  ; Song views:
  ;
  ; 	Add all songs to the playlist
  ;
  ; Properties view:
  ;
  ; 	Add a new property
  ;
  ; Directory View:
  ;
  ; 	Add song(s) to the media library
  (binding A (char->uint #\A)
	   (cond
	     [(eq? system-state 'directory)
	      (directory-media-add-object object)]
	     [(eq? system-state 'props)
	      (add-prop object)]
	     [else
	       (song-views (lambda () (playlist-add-all)))]))

  ; T key:
  ;
  ; Song views:
  ;
  ; 	Manage tags
  (binding T (char->uint #\T)
	   (song-views (lambda () (tags-display object))))

  ; S key:
  ;
  ; Current playlist:
  ; 
  ; 	Randomize/shuffle
  (binding S (char->uint #\S)
	   (cond
	     [(eq? system-state 'current-playlist)
	      (playlist-shuffle)]))

  ; M key:
  ;
  ; Directory view:
  ;
  ; 	Modify directory name
  (binding M (char->uint #\M)
	   (cond
	     [(eq? system-state 'directory)
	      (directory-rename object)]))

  ; R key:
  ;
  ; Song Views:
  ;
  ; 	Change song rating.
  (binding R (char->uint #\R)
	   (song-views (lambda () (ratings-select object))))

  ; L key:
  ;
  ; 	Toggle repeat all
  ;
  (binding L (char->uint #\L)
	   (cond [(string= "1" (result->string (xmmsc_configval_get conn "playlist.repeat_all")))
		  (my-xmms-wait (xmmsc_configval_set conn "playlist.repeat_all" "0"))]
		 [else 
		   (my-xmms-wait (xmmsc_configval_set conn "playlist.repeat_all" "1"))]))

  ; O key:
  ;
  ; 	Toggle repeat one
  ;
  (binding O (char->uint #\O)
	   (cond [(string= "1" (result->string (xmmsc_configval_get conn "playlist.repeat_one")))
		  (my-xmms-wait (xmmsc_configval_set conn "playlist.repeat_one" "0"))]
		 [else 
		   (my-xmms-wait (xmmsc_configval_set conn "playlist.repeat_one" "1"))]))

  ; C key:
  ;
  ; 	Go to collections
  (binding C (char->uint #\C)
	   (menu-selection 'collections))

  ; P key:
  ;
  ; Current Playlist:
  ;
  ; 	Return to previous state
  ;
  ; Everywhere Else:
  ;
  ; 	Go to current playlist
  ;
  (binding P (char->uint #\P)
	   (cond
	     [(eq? system-state 'current-playlist)
	      (revert-state)]
	     [else (menu-selection 'current-playlist)]))

  ; N key:
  ;
  ; All others:
  ;
  ;	Create a new, empty, playlist
  ;
  ; Filter View:
  ;
  ; 	Save as new playlist.
  ;
  ; Directory View:
  ;
  ; 	Create a new directory
  (binding N (char->uint #\N)
	   (cond
	     [(eq? system-state 'filter)
	      (filter-new-collection)]
	     [(eq? system-state 'directory)
	      (directory-new)]
	     [else
	       (new-playlist)]))

  ; U key:
  ;
  ; Playlists:
  ; 
  ;	Use the current or selected playlist.
  ;
  (binding U (char->uint #\U)
	   (cond
	     [(eq? system-state 'playlists)
	      (load-playlist object)]))

  ; D key:
  ;
  ; Playlists:
  ;
  ;	Delete the current or selected playlist.
  ;
  ; Collections:
  ;
  ;	Delete the current or selected collection.
  ;
  ; Directory View:
  ; 	
  ; 	Delete the song
  ;
  (binding D (char->uint #\D)
	   (cond
	     [(eq? system-state 'playlists)
	      (delete-playlist object)]
	     [(and
		(eq? system-state 'collections)
		(eq? collection-type 'collection))
	      (delete-collection object)]
	     [(eq? system-state 'directory)
	      (directory-delete-song object)]))

  ; W key:
  ;
  ; 	Toggle writeback
  (binding W (char->uint #\W)
	   (cond
	     [prop-writeback (set! prop-writeback #f)]
	     [else (set! prop-writeback #t)]))

  ; ? key:
  ;
  ; 	Toggle little help window (bottom pane)
  (binding question (char->uint #\?)
	   (cond
	     [little-help
	       (global-set! 'little-help #f)
	       (make-selection-widget)
	       (refresh-current)]
	     [else
	       (global-set! 'little-help #t)
	       (make-selection-widget)
	       (refresh-current)]))

  ; H key:
  ;
  ; 	Present the big help window.
  (binding H (char->uint #\H)
	   (show-big-help))

  ; Left key:
  ;
  ;	Lots of things involving backing out.
  ;
  (binding left (foreign-value "KEY_LEFT" unsigned-int)
	   (logger "In left_binding: " system-state ", " current-collection ", " current-playlist nl)
	   (cond
	     ; In a particular collection or playlist-as-collection?  Back to the main list
	     [(and
		(eq? system-state 'collections)
		; See if we're in a type that has a current item set
		(or
		  (and
		    (eq? collection-type 'playlist)
		    (not (eq? current-playlist #f)))
		  (and
		    (eq? collection-type 'collection)
		    (not (eq? current-collection #f)))))
	      (display-collection #f)]
	     ; In a particular playlist?  Back to the main list
	     [(and
		(eq? system-state 'playlists)
		playlist-name)
	      (display-playlist #f)]
	     ; Dealing with tags, props, ratings?  Back
	     ; out.
	     [(or
		(eq? system-state 'tags)
		(eq? system-state 'ratings)
		(eq? system-state 'props))
	      (revert-state)]
	     ; Directory: back out
	     [(eq? system-state 'directory)
	      (directory-up)]
	     ; Everywhere else: main menu.
	     [else
	       (show-menu)]))


  ; Right key:
  ;
  ;	Lots of things involving entering, selecting, or adding to
  ;	the playlist.
  ;
  (binding right (foreign-value "KEY_RIGHT" unsigned-int)
	   (logger "In right-binding.\n")
	   (cond
	     ; For the menu, handle the sub-menu item
	     [(eq? system-state 'menu)
	      (handle-menu object)]
	     ; For master collections/playlists list, handle the item
	     [(and
		(eq? system-state 'collections)
		; See if we're in a type that has no current item set
		(or
		  (and
		    (eq? collection-type 'playlist)
		    (eq? current-playlist #f))
		  (and
		    (eq? collection-type 'collection)
		    (eq? current-collection #f))))
	      (display-collection object)]
	     ; For ratings, set the rating
	     [(eq? system-state 'ratings)
	      (ratings-set object)]
	     ; For the playlists list, show the playlist
	     [(eq? system-state 'playlists)
	      (display-playlist object)]
	     ; For the directory list, enter the directory, or play
	     ; the file.
	     [(eq? system-state 'directory)
	      (directory-enter object)]
	     ; Add the column
	     [(eq? system-state 'columns)
	      (columns-add object)]
	     ; Add the tag
	     [(eq? system-state 'tags)
	      (tags-add object)]
	     ; Select the playlist
	     [(eq? system-state 'playlists-selector)
	      (playlists-selector-select object)]
	     ; Edit the prop
	     [(eq? system-state 'props)
	      (props-edit object)]
	     ; Song views: play the song
	     [(and
		(eq? system-state 'collections)
		(or
		  (and
		    (eq? collection-type 'playlist)
		    (not (eq? current-playlist #f)))
		  (and
		    (eq? collection-type 'collection)
		    (not (eq? current-collection #f)))))
	      (playlist-play object)]
	     ; Or in the current playlist view
	     [(eq? system-state 'current-playlist)
	      (playlist-play object)]
	     ; all music view
	     [(eq? system-state 'all-music)
	      (playlist-play object)]
	     ; filter view
	     [(eq? system-state 'filter)
	      (playlist-play object)])
	   (logger "Done right-binding.\n"))

  ; q key:
  ;
  ; Quit
  (binding q (char->uint #\q)
	   (setCDKSelectionPreProcess selection-widget #f #f)
	   (injectCDKSelection selection-widget (foreign-value "KEY_ENTER" unsigned-int))
	   (global-set! 'quit 1))

  )
