(declare (unit collections))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

;; ; Callback functions must be defined in the file they are used.
;; ; *sigh*  At least they can be stubs.
;; (include "collections-callbacks.scm")

; Go from wherever to the collections list
;
; Refresh the widget with either the global list of collections or a
; single one.
(define (show-collections)
  (logger "In show-collections: " collection-type ", " current-collection ", " current-playlist "\n")
  (cond
    ; If our current collection type is unset, get the whole list.
    [(equal? collection-type #f)
     (show-collection-list)]
    ; If the type we're about to get is not set, get the whole list.
    [(and
       (equal? collection-type 'collection)
       (equal? current-collection #f))
     (show-collection-list)]
    [(and
       (equal? collection-type 'playlist)
       (equal? current-playlist #f))
     (show-collection-list)]
    [else
      (logger "About to show coll.\n")
      (show-collection)
      (logger "Done showing coll.\n")]))

; Show the master list of collections.
(define (show-collection-list)
  (logger "In show-collection-list: " current-collection-ns ", " current-collection nl)
  (let*
    ([result (my-xmms-wait (xmmsc_coll_list conn current-collection-ns))]
     [null1 (logger "Result: " result nl)]
     ; Drop all collections starting with _; those are system hidden
     ; collections.
     [colls (remove
	      (lambda (x) (eq? #\_ (string-ref x 0)))
	      ; Turn the result into a list of strings
	      (result->string-list result))])

    (logger "colls: " colls nl)
    (set-selection
      (fmt #f (fit maxwidth
		   (cond
		     [(string= (foreign-value "XMMS_COLLECTION_NS_PLAYLISTS" c-string) current-collection-ns)
		      "   Playlists"]
		     [else "   Collections"])))
      colls
      colls)
    (show-collections-help)))

; Show a particular collection
(define (show-collection)
  (logger "In show-collections: " current-collection-ns ", " current-collection nl)
  (let* (
	 [null1 (logger "About to xmmsc_coll_new.\n")]
	 ; Get a collection reference
	 [collref (xmmsc_coll_new (foreign-value "XMMS_COLLECTION_TYPE_REFERENCE" int))]
	 [null3 (logger "Loading a particular collection; " collref nl)]
	 ; Define the name and namespace for the collection ref
	 [null4 (xmmsc_coll_attribute_set collref "reference"
					  ; Pick our current
					  ; playlist or collection
					  ; name, as appropriate.
					  (cond
					    [(equal? collection-type 'playlist)
					     current-playlist]
					    [(equal? collection-type 'collection)
					     current-collection]))]
	 [null5 (xmmsc_coll_attribute_set collref "namespace" current-collection-ns)]
	 ; Get the ids from the collection
	 [result (my-xmms-wait (xmmsc_coll_query_ids conn collref #f 0 0))]
	 [null1 (logger "Result: " result nl)]
	 [colls (result->int-list result)])

    (logger "colls; media ids: " colls nl)

    ; Set the title
    (cond
      [(equal? collection-type 'playlist)
       (global-set! 'status-extra (conc "Playlist: " current-playlist))]
      [(equal? collection-type 'collection)
       (global-set! 'status-extra (conc "Collection: " current-collection))])

    ; Show the songs
    (show-media colls highlight-current-playlist-good)

    (logger "show-media colls done\n")

    ; Cleanup
    (xmmsc_result_unref result)

    (logger "Done cleanup.\n")
    (show-collection-help)))

; Display just the selected collection
(define (display-collection object)
  (logger "In display-collection: " object nl)
  (cond
    ; If we're going to the general collection list, do so.
    [(equal? object #f)
     (logger "In display-collection, #f is found.\n")
     (global-set! 'current-collection #f)
     (global-set! 'current-playlist #f)
     (logger "In display-collection, global set done.\n")
     (show-collections)
     (logger "In display-collection, refresh done.\n")
     1]
    ; Else get the name of the collection we're entering
    [else
      (define-external obj2 c-pointer object)
      (let*
	([id
	   (list-ref
	     selection-items
	     (getCDKSelectionCurrent selection-widget))]
	 [null1 (logger nl nl nl "open collection event: " (wrt id) nl)])
	(global-set! 
	  ; Set our current playlist or current collection, as
	  ; appropriate.
	  (cond
	    [(equal? collection-type 'playlist)
	     'current-playlist]
	    [(equal? collection-type 'collection)
	     'current-collection])
	  id)
	(show-collections)
	1)]))

; Delete a collection (either the one under the cursor, or the current
; one).  This is only called in true collection mode.
(define (delete-collection object)
  (let* ([to-remove
	   (cond
	     ; If we're in a particular collection, delete that one
	     [current-collection
	       current-collection]
	     ; Otherwise, find which collection we're supposed to load
	     [else
	       (define-external obj2 c-pointer object)
	       (list-ref
		 selection-items
		 (getCDKSelectionCurrent selection-widget))])])
    (logger "In delete-collection: removing " to-remove nl)
    (my-xmms-wait-toss (xmmsc_coll_remove conn to-remove (foreign-value "XMMS_COLLECTION_NS_COLLECTIONS" c-string)))
    ; No matter what, we *certainly* aren't in any particular
    ; collection anymore.
    (global-set! 'current-collection #f)
    (show-collections)))
