(declare (unit menu))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

; Go from wherever to the main menu
(define (show-menu)
  (logger "In show-menu.\n")
  (global-set! 'system-state 'menu)
  (global-set! 'refresh-current (refresh-with show-menu))
  ; Clear other globals out; we want a fresh start here.
  (global-set! 'collection-type #f)
  (global-set! 'playlist-name #f)
  (show-menu-help)
  (logger "In show-menu: about to set-selection.\n")
  (set-selection
    (fmt #f (fit maxwidth "Main Menu"))
    (list
      'current-playlist
      'filter
      ;'artist
      'directory
      'playlists
      'collections
      'all-music
      'playlists-coll
      'columns
      'sort)
    (list
      "Current Playlist"
      "All Music - Filtered (AKA Search)"
      ;"Artist -> Album View"
      "Directory View"
      "Playlists (as playlists)"
      "Collections"
      "All Music (takes a while to load)"
      "Playlists (as collections)"
      "Configure Columns"
      "Configure Sorting")))

; Deal with a menu selection
(define (handle-menu object)
  (logger "In handle-menu " object nl)
  (define-external obj2 c-pointer object)
  (let*
    ([id
       (list-ref
	 selection-items
	 ; Add 1 because the first thing in infos is the labels :)
	 (getCDKSelectionCurrent selection-widget))]
     [null1 (logger nl nl nl "menu id: " (wrt id) nl)])
    (menu-selection id)))

; Deal with a menu selection in a more accessible manner: takes a
; value from the list of selection-items and runs the menu command
; for that.  Used for key bindings like P.
(define (menu-selection id)
  (logger "In menu-selection: " id nl)
  (global-set! 'previous-state system-state)
  (global-set! 'refresh-previous refresh-current)
  (cond
    [(eq? id 'all-music)
     (global-set! 'system-state id)
     (global-set! 'refresh-current (refresh-with show-all-music))
     (show-all-music)]
    ; This is one case where we do not set globals, because a bad
    ; filter means we don't actually enter this mode, and breaking
    ; the pattern by setting the variables inside the menu entry is
    ; easier than recovering what we had before.
    [(eq? id 'filter)
     (filter-all-music)]
    [(eq? id 'current-playlist)
     (global-set! 'system-state id)
     (global-set! 'refresh-current (refresh-with show-current-playlist))
     (show-current-playlist)]
    [(eq? id 'playlists)
     (global-set! 'current-collection-ns
		  (foreign-value "XMMS_COLLECTION_NS_PLAYLISTS" c-string))
     (global-set! 'system-state 'playlists)
     (global-set! 'refresh-current (refresh-with show-playlists))
     (show-playlists)]
    [(eq? id 'directory)
     (global-set! 'system-state id)
     (global-set! 'refresh-current (refresh-with show-directory))
     (show-directory)]
    [(eq? id 'playlists-coll)
     ; Actually a type of collections
     (global-set! 'system-state 'collections)
     (global-set! 'current-collection-ns
		  (foreign-value "XMMS_COLLECTION_NS_PLAYLISTS" c-string))
     (global-set! 'collection-type 'playlist)
     (global-set! 'refresh-current (refresh-with show-collections))
     (show-collections)]
    [(eq? id 'collections)
     (global-set! 'system-state id)
     (global-set! 'current-collection-ns
		  (foreign-value "XMMS_COLLECTION_NS_COLLECTIONS" c-string))
     (global-set! 'collection-type 'collection)
     (global-set! 'refresh-current (refresh-with show-collections))
     (show-collections)]
    [(eq? id 'sort)
     (global-set! 'system-state 'columns)
     (global-set! 'refresh-current (refresh-with (lambda () (configure-columns 'sort-columns (lambda () sort-columns)))))
     (configure-columns 'sort-columns (lambda () sort-columns))]
    [(eq? id 'columns)
     (global-set! 'system-state id)
     (global-set! 'refresh-current (refresh-with (lambda () (configure-columns 'music-columns (lambda () music-columns)))))
     (configure-columns 'music-columns (lambda () music-columns))]
    [else 1])
  1)

; Revert to a previous state
(define (revert-state)
  (logger "In revert-state: " system-state ", " previous-state nl)
  (global-set! 'system-state previous-state)
  (global-set! 'refresh-current refresh-previous)
  (refresh-previous))
