(declare (unit tags))

(use posix)

(include "logger.sci")

#>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

(define tags-song #f)

; Display the tags list for the selected song.
(define (tags-display object)
  (let*
    (
     [list-id
       (getCDKSelectionCurrent object)]
     [id
       (list-ref selection-items list-id)])
    (global-set! 'previous-state system-state)
    (global-set! 'refresh-previous refresh-current)
    (set! tags-song id)
    (global-set! 'system-state 'tags)
    (global-set! 'refresh-current tags-refresh)
    (refresh-current)))

(define (tags-refresh)
  (logger "In tags-refresh: tags-song: " tags-song nl)
  (let*
    (
     ; Get a list of which tags this song is using
     [used
       (filter
	 (lambda (tag)
	   (and
	     (not
	       (string=
		 no-data
		 (first (get-mediainfo tags-song (list tag)))))
	     (not
	       (string=
		 "0"
		 (first (get-mediainfo tags-song (list tag)))))))
	 all-tags)]
     ; Get a list of which tags this song specifically isn't
     [eschew
       (filter
	 (lambda (tag)
	   (string=
	     "0"
	     (first (get-mediainfo tags-song (list tag)))))
	 all-tags)]
     ; Get all the tags that this song isn't already using
     [unused (lset-difference
	       string=
	       (lset-difference
		 string=
		 all-tags
		 used)
	       eschew)]
     ; The scheme selection list (i.e. the raw data)
     [raw-list
       (append
	 used
	 unused
	 eschew)]
     ; The actual text to display.
     [selection-text
       (append
	 (map
	   (lambda (item)
	     (conc good-highlight item))
	   used)
	 unused
	 (map
	   (lambda (item)
	     (conc bad-highlight item))
	   eschew))]
     )
    (logger "In tags-display: used: " used nl)
    (logger "In tags-display: selection-text: " selection-text nl)
    (show-tags-help)
    ; The list here is the current columns, in order, followed
    ; by the rest.
    (set-selection
      (fmt #f (fit maxwidth "Manage Tags"))
      raw-list
      ; If this list is empty, make it un-empty, or CDKSelection
      ; will crash
      (cond
	[(or
	   (not selection-text)
	   (eq? selection-text '()))
	 '("*NO TAGS*")]
	[else
	  selection-text]))))

; Add the selected tags to the song
(define (tags-add object)
  (logger "In tags-add.\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of tags
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each id (now a raw tag name string), set that tag to 1 on
    ; this song
    (lambda (tag)
      (logger "In tags-add: " tag nl)
      (xmmsc_medialib_entry_property_set_int conn tags-song tag 1)))
  (refresh-current))

; Remove the selected tags from the song
(define (tags-remove object)
  (logger "In tags-remove.\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of tags
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each id (now a raw tag name string), set that tag to 1 on
    ; this song
    (lambda (tag)
      (logger "In tags-remove: " tag nl)
      (xmmsc_medialib_entry_property_set_int conn tags-song tag 0)))
  (refresh-current))
