(use libxmmsclient)
(use lolevel)

(define (return-mediainfo conn id)
  ;(define-external val c-string-list)
  (define-external val c-string "N/A")
  (define intval (s32vector 999999))
  (define result (xmmsc_medialib_get_info conn id))
  (xmmsc_result_wait result)

  ; Is this an error?
  (and (= (xmmsc_result_iserror result) 1)
       (begin
	 (display (conc
		    "medialib get info returns error, "
		    (xmmsc_result_get_error result)
		    "\n"))))
  (define final-result
    (list
      (cond
	((= 0 (xmmsc_result_get_dict_entry_string result "artist" (location val)))
	 (quote "artist = No artist\n"))
	(else
	  (conc "artist = " val "\n")))

      (cond
	((= 0 (xmmsc_result_get_dict_entry_string result "title" (location val)))
	 (quote "title = No title\n"))
	(else
	  (conc "title = " val "\n")))

      (cond
	((= 0 (xmmsc_result_get_dict_entry_string result "url" (location val)))
	 (quote "url = No url\n"))
	(else
	  (conc "url = " val "\n")))

      (cond
	((= 0 (xmmsc_result_get_dict_entry_int result "bitrate" intval))
	 (quote "bitrate = No bitrate\n"))
	(else
	  (conc "bitrate = " (s32vector-ref intval 0) "\n")))))

  (xmmsc_result_unref result)

  final-result
  )

(define (print-mediainfo conn id)
  ;(define-external val c-string-list)
  (define-external val c-string "N/A")
  (define intval (s32vector 999999))
  (define result (xmmsc_medialib_get_info conn id))
  (xmmsc_result_wait result)

  ; Is this an error?
  (and (= (xmmsc_result_iserror result) 1)
       (begin
	 (display (conc
		    "medialib get info returns error, "
		    (xmmsc_result_get_error result)
		    "\n"))))

  (display "Getting artist.\n")
  (cond
    ((= 0 (xmmsc_result_get_dict_entry_string result "artist" (location val)))
     (display "No artist\n"))
    (else
      (display (conc "artist = " val "\n")))
    )
  (display "Done getting artist.\n")

  (cond
    ((= 0 (xmmsc_result_get_dict_entry_string result "title" (location val)))
     (display "No title\n"))
    (else
      (display (conc "title = " val "\n")))
    )

  (cond
    ((= 0 (xmmsc_result_get_dict_entry_int result "bitrate" intval))
     (display "No bitrate\n"))
    (else
      (display (conc "bitrate = " (s32vector-ref intval 0) "\n")))
    )

  (xmmsc_result_unref result)
  )

;
; The first part of this program is
; commented in tut1.c See that one for
; instructions
;
(define conn (xmmsc_init "tut2"))

(xmmsc_connect conn "unix:///tmp/xmms-ipc-rlpowell")
;
; Now we send a command that will return
; a result. Let's find out which entry
; is currently playing. 
;
; Note that this program has be run while 
; xmms2 is playing something, otherwise
; xmmsc_playback_current_id will return 0.
;
(define result (xmmsc_playback_current_id conn))

;
; We are still doing sync operations, wait for the
; answer and block.
;
(xmmsc_result_wait result)

;
; Also this time we need to check for errors.
; Errors can occur on all commands, but not signals
; and broadcasts. We will talk about these later.
;
(display (conc "result is error: " (xmmsc_result_iserror result) "\n"))

;
; Let's retrieve the value from the result struct.
; The caveat here is that you have to know what type
; of value is returned in response to each command.
;
; In this case we know that xmmsc_playback_current_id
; will return a UINT
;
; Know that all xmmsc_result_get calls can return FALSE
; and that means that the value you are requesting is
; not in the result struct.
;
; Values are stored in the pointer passed to result_get
;



;(define-external id u32vector 7823412)
(define id (u32vector 1234567))
(display "before get_uint.\n")
(xmmsc_result_get_uint result id)
(display "after get_uint.\n")
(display (conc "uint: " (u32vector-ref id 0) "\n"))

(display "after let-location.\n")
(print-mediainfo conn (u32vector-ref id 0))

(xmmsc_result_unref result)

(display "After first print.\n")

; Get the full playlist
(define result2 (xmmsc_playlist_list_entries conn #f))
(xmmsc_result_wait result2)

;(define (playlist-loop result)
;  (define valid (xmmsc_result_list_valid result))
;  (display (conc "valid: " valid "\n"))
;  (cond
;    ((= 1 valid)
;     (define id (u32vector 9999999))
;     (xmmsc_result_get_uint result id)
;     (display (conc "uint: " (first (u32vector->list id)) "\n"))
;     (define info (return-mediainfo conn (first (u32vector->list id))))
;     (display (conc "info: " info "\n"))
;     ; Go to the next item
;     (define next-ret (xmmsc_result_list_next result))
;     (display (conc "next-ret: " next-ret "\n"))
;     (list info (playlist-loop result)))))
;
;(display (conc "loop: " (playlist-loop result2) "\n"))

; Returns a list of all ids in the current playlist
(define (playlist result)
  (let* ([id (u32vector 9999999)]
	 [valid (xmmsc_result_list_valid result)]
	 )
    ;(display (conc "valid: " valid "\n"))
    (cond
      ((= 1 valid)
       ; Get the id of the current playlist bit
       (xmmsc_result_get_uint result id)
       ;(display (conc "uint: " (u32vector-ref id 0) "\n"))
       ; Go to the next item
       (xmmsc_result_list_next result)
       (cons (u32vector-ref id 0) (playlist result)))
      (else '()))))

(define ids (playlist result2))
(display (conc "ids: " ids "\n"))

(define infos (map (lambda (x) (return-mediainfo conn x)) ids))

;(display "infos: ")
;(write (map (lambda (x) (return-mediainfo conn x)) ids))
;(display "\n")

;(display (conc "infos: " (map (lambda (x) (return-mediainfo conn x)) ids) "\n"))

(display (conc "first: " (first infos)))

(xmmsc_result_unref result2)

(xmmsc_unref conn)

; *************************
; Now we try CDK bits.
; ******

#>
#include <curses.h>
#include <cdk/cdk.h>

CDK_PARAMS params;
<#

#>?

typedef struct _win_st WINDOW;
typedef struct SScreen CDKSCREEN;
typedef struct CDK_PARAMS CDK_PARAMS;

WINDOW *initscr(void);
CDKSCREEN *initCDKScreen (WINDOW *cursesWindow);
void initCDKColor (void) ;

CDKITEMLIST *newCDKItemlist (
               CDKSCREEN *cdkscreen,
               int xpos,
               int ypos,
               char *title,
               char *label,
               char **itemList,
               int itemCount,
               int defaultItem,
               int box,
               int shadow);

typedef struct SItemList CDKITEMLIST;

#define  CENTER          9002

int CDKparamValue (
               CDK_PARAMS * params,
               int option,
               int missing);

int activateCDKItemlist (
               CDKITEMLIST *itemlist,
               chtype *actions);

typedef unsigned long chtype;
typedef int boolean;

CDKALPHALIST *newCDKAlphalist (
               CDKSCREEN *cdkscreen,
               int xpos,
               int ypos,
               int height,
               int width,
               char * title,
               char * label,
               char ** list,
               int listSize,
               chtype fillerCharacter,
               chtype highlight,
               boolean box,
               boolean shadow);

char *activateCDKAlphalist (
               CDKALPHALIST *alphalist,
               chtype *actions);

void destroyCDKAlphalist (
               CDKALPHALIST *alphalist);

void destroyCDKScreen(
               CDKSCREEN *screen);

void endCDK(void) ;

<#

(foreign-code "CDKparseParams(C_main_argc, C_main_argv, &params, \"cNSX:Y:\");")

#>
char **make_strings(C_word lst)
{
   int len, i;
   C_word tmp;
   char **hlist;

printf( "in make_strings.\n" );
printf( "in make_strings2: %d.\n",C_unfix(C_i_length(lst)) );

   len = C_unfix(C_i_length(lst));
   hlist = (char**) malloc(sizeof(char*) * (len + 1));

   for(i = 0, tmp = lst; i < len; i++, tmp = C_u_i_cdr(tmp)) {
      hlist[i] = C_c_string(C_u_i_car(tmp));
	printf( "str: %d, %s.\n", i, hlist[i] );
   }
   hlist[i] = NULL;

   return hlist;
}
<#

#>!
char **make_strings(C_word lst);
<#

(display "pre-make.\n")
(define-external infoz c-pointer (make_strings (first infos)))
(display "post-make\n")

;(display "make window.\n")
(define curses-window (initscr))
;(display "make screen.\n")
(define cdk-screen (initCDKScreen curses-window))

;(display "init color.\n")
(initCDKColor)


(define infoslist
  (newCDKAlphalist
    cdk-screen
    CENTER
    CENTER
    0
    0
    "<C>Pick A Month"
    "</U/5>Month:"
    infoz
    (length (first infos))
    (foreign-value "(unsigned int) '_'" unsigned-int)
    (foreign-value "A_REVERSE" unsigned-int)
    1
    1))

;(display "\n\n\n")
;(display (conc "list: " infoslist "\n"))

(define choice (activateCDKAlphalist infoslist #f))

; Clean up.
(destroyCDKAlphalist infoslist);
(destroyCDKScreen cdk-screen);
(endCDK);

(display (conc "choice: " choice "\n"))
